from pathlib import Path
import fbc_curation
from cobra.io import read_sbml_model
from fbc_curation import examples, fbc_files
from fbc_curation import EXAMPLE_PATH

model_path = EXAMPLE_PATH / "models" / "e_coli_core.xml"


def test_objective_value(tmp_path):
    model = read_sbml_model(str(model_path))
    results_path = tmp_path / "test.tsv"
    obj_value = fbc_files.create_objective_file(model, results_path)
    assert isinstance(obj_value, float)
    assert obj_value > 0
    assert Path.exists(results_path)


def test_fva(tmp_path):
    model = read_sbml_model(str(model_path))
    results_path = tmp_path / "test.tsv"
    df = fbc_files.create_fva_file(model, results_path)
    assert not df.empty
    assert Path.exists(results_path)

    assert len(df) == len(model.reactions)
    assert "reaction" in df.columns
    assert "minimum" in df.columns
    assert "maximum" in df.columns


def test_gene_deletion(tmp_path):
    model = read_sbml_model(str(model_path))
    results_path = tmp_path / "test.tsv"
    df = fbc_files.create_gene_deletion_file(model, results_path)
    assert not df.empty
    assert Path.exists(results_path)

    assert len(df) == len(model.genes)
    assert "gene" in df.columns
    assert "value" in df.columns
    assert "status" in df.columns


def test_reaction_deletion(tmp_path):
    model = read_sbml_model(str(model_path))
    results_path = tmp_path / "test.tsv"
    df = fbc_files.create_reaction_deletion_file(model, results_path)
    assert not df.empty
    assert Path.exists(results_path)

    assert len(df) == len(model.reactions)
    assert "reaction" in df.columns
    assert "value" in df.columns
    assert "status" in df.columns


def test_e_coli_core(tmp_path):
    examples.example_ecoli_core(tmp_path)
    assert Path.exists(tmp_path / "e_coli_core_01_objective.tsv")
    assert Path.exists(tmp_path / "e_coli_core_02_fva.tsv")
    assert Path.exists(tmp_path / "e_coli_core_03_gene_deletion.tsv")
    assert Path.exists(tmp_path / "e_coli_core_04_reaction_deletion.tsv")


def test_iJR904(tmp_path):
    examples.example_iJR904(tmp_path)
    assert Path.exists(tmp_path / "iJR904_01_objective.tsv")
    assert Path.exists(tmp_path / "iJR904_02_fva.tsv")
    assert Path.exists(tmp_path / "iJR904_03_gene_deletion.tsv")
    assert Path.exists(tmp_path / "iJR904_04_reaction_deletion.tsv")



"""
Create curation information for example models.
"""

from fbc_curation import EXAMPLE_PATH
from fbc_curation.fbc_files import create_fbc_files


def example_ecoli_core(results_dir):
    """Create example files for ecoli core."""
    model_path = EXAMPLE_PATH / "models" / "e_coli_core.xml"
    create_fbc_files(results_dir=results_dir, model_path=model_path)


def example_iJR904(results_dir):
    """Create example files for ecoli core."""
    model_path = EXAMPLE_PATH / "models" / "iJR904.xml.gz"
    create_fbc_files(results_dir=results_dir, model_path=model_path)


def run_examples():
    results_dir = EXAMPLE_PATH / "results"
    example_ecoli_core(results_dir)
    example_iJR904(results_dir)


if __name__ == "__main__":
    run_examples()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
sbmlutils pip package
"""
import io
import re
import os
from setuptools import find_packages
from setuptools import setup

setup_kwargs = {}


def read(*names, **kwargs):
    """ Read file info in correct encoding. """
    return io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


# version from file
verstrline = read('fbc_curation/_version.py')
mo = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", verstrline, re.M)
if mo:
    verstr = mo.group(1)
    setup_kwargs['version'] = verstr
else:
    raise RuntimeError("Unable to find version string")

# description from markdown
long_description = read('README.rst')
setup_kwargs['long_description'] = long_description

setup(
    name='fbc_curation',
    description='FBC reference files for SBML model curation using cobrapy',
    url='https://github.com/matthiaskoenig/fbc_curation',
    author='Matthias König',
    author_email='konigmatt@googlemail.com',
    license='LGPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Cython',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Bio-Informatics'
    ],
    keywords='SBML, FBC, simulation',
    packages=find_packages(),
    # package_dir={'': ''},
    package_data={
      '': [
              '../requirements.txt',
               '../examples',
          ],
    },
    include_package_data=True,
    zip_safe=False,
    python_requires='>=3.6',
    # List run-time dependencies here.  These will be installed by pip when
    install_requires=[
        "pandas",
        "cobra",
        "pytest",
        "pytest-cov"
    ],
    entry_points={
        'console_scripts':
            [
                'fbc_curation_examples=fbc_curation.examples:run_examples',
                'fbc_curation=fbc_curation.fbc_files:main',
            ],
    },
    extras_require={},
    **setup_kwargs)



#!/bin/sh
"""
Module for creating FBC curation files via cobrapy.

Uses GLPK as default solver.
"""
import pandas as pd
import cobra
from cobra.io import read_sbml_model
from pathlib import Path

from cobra.flux_analysis import flux_variability_analysis
from cobra.flux_analysis import single_gene_deletion, single_reaction_deletion

NUM_DECIMALS = 8  # decimals to write in the solution


def print_header(title):
    print()
    print("-" * 80)
    print(title)
    print("-" * 80)


def create_fbc_files(model_path: Path, results_dir, decimals=NUM_DECIMALS) -> pd.DataFrame:
    """ Creates all FBC curation files

    Model is reloaded for different files to make sure
    simulations are run from the initial model state.

    :param model_path:
    :param results_dir:
    :return:
    """
    # objective value
    mpath = str(model_path)
    model = read_sbml_model(mpath)
    mid = model.id
    create_objective_file(model, results_dir / f"{mid}_01_objective.tsv",
                          decimals=decimals)

    # flux variability
    mpath = str(model_path)
    model = read_sbml_model(mpath)
    mid = model.id
    create_fva_file(model, results_dir / f"{mid}_02_fva.tsv",
                    decimals=decimals)

    # gene deletions
    model = read_sbml_model(mpath)
    mid = model.id
    create_gene_deletion_file(model, results_dir / f"{mid}_03_gene_deletion.tsv",
                              decimals=decimals)

    # reaction deletions
    model = read_sbml_model(mpath)
    mid = model.id
    create_reaction_deletion_file(model, results_dir / f"{mid}_04_reaction_deletion.tsv",
                                  decimals=decimals)


def create_objective_file(model: cobra.Model, path: Path, decimals: int=NUM_DECIMALS) -> float:
    """ Creates TSV file with objective value.

    see https://cobrapy.readthedocs.io/en/latest/simulating.html
    :param model:
    :param path:
    :return:
    """
    print_header(f"Objective: {path}")
    # create data frame and store
    solution = model.optimize()
    print(solution)
    obj_value = round(solution.objective_value, NUM_DECIMALS)

    with open(path, "w") as f_out:
        f_out.write(str(obj_value))

    return obj_value


def create_fva_file(model: cobra.Model, path: Path, decimals: int=NUM_DECIMALS) -> pd.DataFrame:
    """ Creates TSV file with minimum and maximum value of Flux variability analysis.

    see https://cobrapy.readthedocs.io/en/latest/simulating.html#Running-FVA
    :param model:
    :param path:
    :return:
    """
    print_header(f"FVA: {path}")
    # create data frame and store
    df = flux_variability_analysis(model, model.reactions)

    # clean DataFrame
    df_out = pd.DataFrame(
        {"reaction": df.index,
         "minimum": df.minimum,
         "maximum": df.maximum
         })
    for key in ["minimum", "maximum"]:
        df_out[key] = df_out[key].apply(lambda x: round(x, decimals))
    df_out.sort_values(by=['reaction'], inplace=True)
    df_out.index = range(len(df_out))
    print(df_out.head(10))
    print('...')

    df_out.to_csv(path, sep="\t", index=False)
    return df_out


def create_gene_deletion_file(model: cobra.Model, path: Path,
                              decimals: int=NUM_DECIMALS) -> pd.DataFrame:
    """ Creates TSV with results of gene deletion.

    https://cobrapy.readthedocs.io/en/latest/deletions.html
    :param model:
    :param path:
    :return:
    """
    print_header(f"Gene deletions: {path}")
    # create data frame and store
    df = single_gene_deletion(model, model.genes)

    # clean DataFrame
    df_out = pd.DataFrame(
        {"gene": [set(ids).pop() for ids in df.index],
         "value": df.growth,
         "status": df.status
         })
    df_out.index = range(len(df_out))
    for key in ["value"]:
        df_out[key] = df_out[key].apply(lambda x: round(x, decimals)).abs()  # abs fixes the -0.0 | +0.0 diffs
    df_out.sort_values(by=['gene'], inplace=True)
    print(df_out.head(10))
    print('...')

    df_out.to_csv(path, sep="\t", index=False)
    return df_out


def create_reaction_deletion_file(model: cobra.Model, path: Path,
                                  decimals: int=NUM_DECIMALS) -> pd.DataFrame:
    """ Creates TSV with results of reaction deletion.

    https://cobrapy.readthedocs.io/en/latest/deletions.html
    :param model:
    :param path:
    :return:
    """
    print_header(f"Reaction deletions: {path}")
    # create data frame and store
    df = single_reaction_deletion(model, model.reactions)

    # clean DataFrame
    df_out = pd.DataFrame(
        {"reaction": [set(ids).pop() for ids in df.index],
         "value": df.growth,
         "status": df.status
         })
    df_out.sort_values(by=['reaction'], inplace=True)
    for key in ["value"]:
        df_out[key] = df_out[key].apply(lambda x: round(x, decimals)).abs()  # abs fixes the -0.0 | +0.0 diffs
    df_out.index = range(len(df_out))
    print(df_out.head(10))
    print('...')

    df_out.to_csv(path, sep="\t", index=False)
    return df_out


def main():
    """
     Example:
         python fbc_curation.py --model ./models/e_coli_core.xml --out ./results
     """
    import sys
    import optparse
    parser = optparse.OptionParser()
    parser.add_option('-m', '--model',
                      action="store", dest="model_path",
                      help="path to SBML model with fbc information")
    parser.add_option('-o', '--out',
                      action="store", dest="output_path",
                      help="path to write the output to")

    options, args = parser.parse_args()

    if not options.model_path:
        print("Required argument '--model' missing")
        parser.print_help()
        sys.exit(1)
    if not options.output_path:
        print("Required argument '--out' missing")
        parser.print_help()
        sys.exit(1)

    model_path = Path(options.model_path)
    output_path = Path(options.output_path)
    create_fbc_files(results_dir=output_path, model_path=model_path)

if __name__ == "__main__":
    main()
